; =================================
; MERLIN/BIGMAC & MONITOR for Oric
; quick & dirty wrapper loader
; copyleft iss (@) 2015
; =================================

*       = LOADER

_himem         = $00a6

_putch         = $0238
_getch         = $023b
_lprint        = $023e

_flags         = $026a
_currow        = $0268
_curcol        = $0269
_keydelay      = $024e
_keylast       = $02df
_gparm1        = $02e1
_gparm2        = $02e2

_exvec         = $02f5

_ink           = $f210
_paper         = $f204
_clrsts        = $e5f5
_rstbas        = $c4a8

_dosrom        = $04f2     ; toggle rom on/off
_doserr        = $04fd     ; dos error

bigmac_entry  = $08f6
monitor_entry = $9104

;
;
;
ldr
.(
        ; Move monitor code at $9000
        jsr move_monitor

        ; Zero the area %0500 - $0550
        lda #0
        tax
loop
        sta $0500,x
        inx
        cpx #$50
        bne loop

        ; Update himem to $0550
        lda _himem
        sta hmem_lo
        lda _himem+1
        sta hmem_hi

        lda #<$0550
        sta _himem
        lda #>$0550
        sta _himem+1

        ; Get ! vector
        lda _exvec
        sta dos_pt+1
        lda _exvec+1
        sta dos_pt+2

        ; Hook lprint vector
        jsr lpr_catch

        ; Hook monitor exit routine
        lda #$4c
        sta $9184
        lda #<quit_hook_mon
        sta $9184+1
        lda #>quit_hook_mon
        sta $9184+2

        ; Hook bigmac exit routine
        lda #$4c
        sta $0B06
        lda #<quit_hook_bm
        sta $0B06+1
        lda #>quit_hook_bm
        sta $0B06+2

        ; Apply patches to bigmac:
        ; = loaded file size
        ; 0A57: LDY byte_7AA0
        ; 0A5A: LDX byte_7A9F
        lda #<(_sed_size+1)
        sta $0a57+1
        lda #>(_sed_size+1)
        sta $0a57+2
        lda #<(_sed_size)
        sta $0a5a+1
        lda #>(_sed_size)
        sta $0a5a+2
        ; = link to monitor
        ; 05B6: LDA MON_9000
        lda #$4c
        sta $05b6
        lda #<$05c7
        sta $05b7
        lda #>$05c7
        sta $05b8
        ; = no key click attribute
        ; 05C7: LDA #3
        lda #$0b
        sta $05c8
        ; = link to monitor code
        ; 05CC: JMP MON_9000
        lda #<monitor_entry
        sta $05cd
        lda #>monitor_entry
        sta $05ce

        ; = object code start hack
        ; 0B79: JSR TMPJMP_8FF3
        lda #$4c
        sta $0b79
        lda #$28
        sta $0b7a
        lda #$09
        sta $0b7b

        ; Store current memory state
        sei
        jsr clrscrn
        jsr mem_store
        cli

        jsr init_vars

s        ; Start bigmac
        jmp bigmac_entry
.)

dos_pt  jmp $1234

hitkey  .asc $0d,$0a,$0d,$0a
        .asc "Press 'RETURN' ",$00

hmem_lo .byt 0
hmem_hi .byt 0
cnt     .byt 1
ftyp    .byt 1
save_a  .byt 0
save_x  .byt 0
save_y  .byt 0
buf_cmd .dsb 40

;
; Initialize 'local' vars
;
init_vars
.(
        lda #1
        sta cnt
        sta ftyp
        rts
.)

;
; move pages 0,2,4 to buffers
;
mem_store
.(
        ldy #1
        ldx #0

        lda #<$0400
        sta mov_src
        lda #>$0400
        sta mov_src+1

        lda #<buf_0400
        sta mov_dst
        lda #>buf_0400
        sta mov_dst+1

        jsr mem_move

        ldy #1
        ldx #0

        lda #<$0200
        sta mov_src
        lda #>$0200
        sta mov_src+1

        lda #<buf_0200
        sta mov_dst
        lda #>buf_0200
        sta mov_dst+1

        jsr mem_move

        ldy #1
        ldx #0

        lda #<$0000
        sta mov_src
        lda #>$0000
        sta mov_src+1

        lda #<buf_0000
        sta mov_dst
        lda #>buf_0000
        sta mov_dst+1

        jsr mem_move

        rts
.)

;
; move buffers to pages 0,2,4
;
mem_restore
.(
        ldy #1
        ldx #0

        lda #<buf_0400
        sta mov_src
        lda #>buf_0400
        sta mov_src+1

        lda #<$0400
        sta mov_dst
        lda #>$0400
        sta mov_dst+1

        jsr mem_move

        ldy #1
        ldx #0

        lda #<buf_0200
        sta mov_src
        lda #>buf_0200
        sta mov_src+1

        lda #<$0200
        sta mov_dst
        lda #>$0200
        sta mov_dst+1

        jsr mem_move

        ldy #1
        ldx #0

        lda #<buf_0000
        sta mov_src
        lda #>buf_0000
        sta mov_src+1

        lda #<$0000
        sta mov_dst
        lda #>$0000
        sta mov_dst+1

        jsr mem_move

        rts
.)

;
; swap pages 0,2,4 with buffers
;
mem_switch
.(
        ldy #1
        ldx #0

        lda #<buf_0400
        sta swp_src
        lda #>buf_0400
        sta swp_src+1

        lda #<$0400
        sta swp_dst
        lda #>$0400
        sta swp_dst+1

        jsr mem_swap

        ldy #1
        ldx #0

        lda #<buf_0200
        sta swp_src
        lda #>buf_0200
        sta swp_src+1

        lda #<$0200
        sta swp_dst
        lda #>$0200
        sta swp_dst+1

        jsr mem_swap

        ldy #1
        ldx #0

        lda #<buf_0000
        sta swp_src
        lda #>buf_0000
        sta swp_src+1

        lda #<$0000
        sta swp_dst
        lda #>$0000
        sta swp_dst+1

        jsr mem_swap

        rts
.)

;
; return from monitor to bigmac
;
quit_hook_mon
.(
        jsr clrscrn
        jmp bigmac_entry
.)

;
; quit to dos prompt
;
quit_hook_bm
.(
        sei
        jsr mem_restore
        lda hmem_lo
        sta _himem
        lda hmem_hi
        sta _himem+1
        cli

        jsr clrscrn
        jsr flushkey

        jmp _rstbas
.)

;
;
;
flushkey
.(
        jsr _getch
        lda #$00
        sta _keylast
        rts
.)

;
; print message and wait return key
;
waitreturn
.(
        ldy #0
next
        lda hitkey,y
        beq getreturn
        tax
        jsr _putch
        iny
        jmp next

getreturn
        jsr _getch
        beq getreturn
        cmp #$0d
        bne getreturn
        rts
.)

;
; clear screen and more
;
clrscrn
.(
        ; Clear cassette status
        jsr _clrsts

        lda #$2b
        sta _flags
        lda #$20
        sta _keydelay

        lda #0
        sta _gparm2
        sta _gparm1
        jsr _paper
        lda #7
        sta _gparm1
        jsr _ink

        ldx #$0c
        jsr _putch
        rts
.)

;
; hook lprint vector
;
lpr_catch
.(
        lda #<lprint
        sta _lprint+1
        lda #>lprint
        sta _lprint+2
        rts
.)

;
; Reg Y must be preserved in lprint
;
lprint_xxx
.(
        sta save_a
        stx save_x
        sty save_y

        ldx cnt

        cmp #$0a
        bne next
point
        jmp clr

next
        cmp #$0d
        beq exit
        cmp #$02
        beq exit

stor2
        sta $bb80,x
        inx
        stx cnt
        lda #'<'
        sta $bb80,x
        cpx #40
        bne exit

clr
        jsr flushkey
        jsr init_vars

exit
        ldy save_y
        ldx save_x
        lda save_a
        rts
.)


lprint
.(
        sta save_a
        stx save_x
        sty save_y

        ldx cnt

        cmp #$0a
        bne next

        cpx #1
        beq skip
        jsr send_cmd
skip
        jmp clr

next
        cmp #$0d
        beq exit
        cmp #$02
        beq exit

        cmp #','
        bne stor
        lda #0
stor
        cmp #'.'
        bne stor2
        ldy #0
        sty ftyp
stor2
        sta buf_cmd,x
        inx
        stx cnt
        cpx #40
        bne exit

clr
        jsr flushkey
        jsr init_vars

exit
        ldy save_y
        ldx save_x
        lda save_a
        rts
.)

;
; execute dos operation
;
send_cmd
.(
        lda #0
        ldx cnt
        sta buf_cmd,x
        lda buf_cmd+1

do_load
        cmp #'L'
        beq go_load
        jmp do_save

go_load
        lda #<(buf_cmd+5)
        sta _sed_fname
        lda #>(buf_cmd+5)
        sta _sed_fname+1

        sei
        jsr clrscrn
        jsr mem_switch
        jsr clrscrn
        jsr _sed_loadfile
        jsr clrscrn
        jsr mem_switch
        jsr clrscrn
        cli

        inc _sed_size
        bne ok_load
        inc _sed_size+1
ok_load
        rts

do_save
        cmp #'S'
        beq go_save
        jmp do_dir

go_save
        lda #<(buf_cmd+5)
        sta _sed_fname
        lda #>(buf_cmd+5)
        sta _sed_fname+1

        sec
        lda $6e
        sbc $6a
        sta _sed_size
        lda $6f
        sbc $6b
        sta _sed_size+1
        bne ok_save
        lda _sed_size
        bne ok_save
        rts

ok_save
        lda $6a
        sta _sed_begin
        lda $6b
        sta _sed_begin+1

        lda $6e
        sta _sed_end
        lda $6f
        sta _sed_end+1

        ; $e6 - exe addres
        lda $e6
        sta _sed_exe
        lda $e7
        sta _sed_exe+1

        ; $e2 - exe begin
        lda $e2
        sta _sed_exe_begin
        lda $e3
        sta _sed_exe_begin+1

        ; $ec - exe end
        lda $ec
        sta _sed_exe_end
        lda $ed
        sta _sed_exe_end+1

        sei
        jsr clrscrn
        jsr mem_switch
        jsr clrscrn
        lda ftyp
        beq sav_src
        jsr _sed_savefile_exe
        jsr _sed_update_exe
        jmp sav_end
sav_src
        jsr _sed_savefile_src
sav_end
        jsr clrscrn
        jsr mem_switch
        jsr clrscrn
        cli
        rts

do_dir
        sei
        jsr clrscrn
        jsr mem_switch
        jsr clrscrn
        jsr _sed_dir
        cli
        jsr waitreturn
        sei
        jsr clrscrn
        jsr mem_switch
        jsr clrscrn
        cli
        rts
.)

;
; Call execute
; Y:X - poiter to command
;
save_e9 .byt 0
save_ea .byt 0
execute
.(
        lda $e9
        sta save_e9
        lda $ea
        sta save_ea

        ; update the line start pointer
        stx $e9
        sty $ea

        ; get next token
        jsr $00e2
        ; if end of line, go back
        tax
        beq exit

        jsr dos_pt

exit
        lda save_e9
        sta $e9
        lda save_ea
        sta $ea

        rts
.)

;
; relocate monitor code at $9000
;
move_monitor
.(
        lda #<MONITOR_OFFSET
        sta mov_src
        lda #>MONITOR_OFFSET
        sta mov_src+1

        lda #<MONITOR
        sta mov_dst
        lda #>MONITOR
        sta mov_dst+1

        ldx #<MONITOR_SIZE
        ldy #>MONITOR_SIZE
        jsr mem_move
        rts
.)

;
; mem_move Y:X size
;
mem_move
        txa
        pha
        ldx #0
        tya
        beq mem_rest___

mem_loop___
        jsr mem_move___
        dey
        bne mem_loop___

mem_rest___
        pla
        tax
        bne mem_move___
        rts

        ; continue with the rest
mem_move___

mov_src  = (*+1)
        lda $1234
mov_dst  = (*+1)
        sta $1234

        inc mov_src
        bne mms_skip___
        inc mov_src+1

mms_skip___
        inc mov_dst
        bne mmd_skip___
        inc mov_dst+1

mmd_skip___
        dex
        bne mem_move___
        rts

mem_swap
        lda swp_src
        sta swp_src2
        lda swp_src+1
        sta swp_src2+1
        lda swp_dst
        sta swp_dst2
        lda swp_dst+1
        sta swp_dst2+1

        txa
        pha
        ldx #0
        tya
        beq swp_rest___

swp_loop___
        jsr swp_move___
        dey
        bne swp_loop___

swp_rest___
        pla
        tax
        bne swp_move___
        rts

        ; continue with the rest
swp_tmp .byt 0
swp_move___
swp_src = (*+1)
        lda $1234
        sta swp_tmp
swp_dst = (*+1)
        lda $1234
swp_src2= (*+1)
        sta $1234
        lda swp_tmp
swp_dst2= (*+1)
        sta $1234

        inc swp_src
        bne mss_skip___
        inc swp_src+1
mss_skip___
        lda swp_src
        sta swp_src2
        lda swp_src+1
        sta swp_src2+1

        inc swp_dst
        bne msd_skip___
        inc swp_dst+1
msd_skip___
        lda swp_dst
        sta swp_dst2
        lda swp_dst+1
        sta swp_dst2+1

        dex
        bne swp_move___
        rts

; ======================================

; _sed_fname  = pointer to filename
; _sed_begin  = Start address
; _sed_end    = End address
; _sed_size   = Size (not used)

_sed_fname
        .byt 0,0
_sed_begin
        .byt 0,0
_sed_end
        .byt 0,0
_sed_size
        .byt 0,0

_sed_exe
        .byt 0,0
_sed_exe_begin
        .byt 0,0
_sed_exe_end
        .byt 0,0
_sed_exe_size
        .byt 0,0

_sed_savefile_src
.(
        ; -- sei ; disable interrupts
        lda _sed_fname
        sta $e9 ; Filename lo
        lda _sed_fname+1
        sta $ea ; Filename hi

        jsr _dosrom  ; enable OverlayRAM

        lda #$00
        jsr $d454 ; verify filename and copy it to BUFNOM

        ; Setup Areas
        lda _sed_begin
        sta $c052 ; Start Address Lo
        lda _sed_begin+1
        sta $c053 ; Start Address Hi

        lda _sed_end
        sta $c054 ; End Address Lo
        lda _sed_end+1
        sta $c055 ; End Address Hi

        lda _sed_begin
        sta $c056 ; Execution Address Lo
        lda _sed_begin+1
        sta $c057 ; Execution Address Hi

        lda #$00  ; #$00 - SAVEO, #$C0 - SAVEU
        sta $c04d

        lda #$00  ; 0 here means no params
        sta $c04e

        lda #$40  ; file type - data and no auto
        sta $c051

        jsr $de0b ; set LGSAL0 and call XSAVEB

        jsr _dosrom ; disable Overlay RAM
        ; -- cli ; re-enable interrupts

        ldx _doserr ; Get result (low byte)
        lda #0   ; Get result (high byte)
        rts
.)

_sed_savefile_exe
.(
        ; -- sei ; disable interrupts
        lda _sed_fname
        sta $e9 ; Filename lo
        lda _sed_fname+1
        sta $ea ; Filename hi

        jsr _dosrom  ; enable OverlayRAM

        lda #$00
        jsr $d454 ; verify filename and copy it to BUFNOM

        ;Setup Areas
        lda _sed_exe_begin
        sta $c052 ; Start Address Lo
        lda _sed_exe_begin+1
        sta $c053 ; Start Address Hi

        lda _sed_exe_end
        sta $c054 ; End Address Lo
        lda _sed_exe_end+1
        sta $c055 ; End Address Hi

        lda _sed_exe
        sta $c056 ; Execution Address Lo
        lda _sed_exe+1
        sta $c057 ; Execution Address Hi

        lda #$00  ; #$00 - SAVEO, #$C0 - SAVEU
        sta $c04d

        lda #$00  ; 0 here means no params
        sta $c04e

        lda #$41  ; file type - program and auto
        sta $c051

        jsr $de0b ; set LGSAL0 and call XSAVEB

        jsr _dosrom ; disable Overlay RAM
        ; -- cli ; re-enable interrupts

        ldx _doserr ; Get result (low byte)
        lda #0   ; Get result (high byte)
        rts
.)

_sed_loadfile
.(
        ; -- sei ; disable interrupts
        lda _sed_fname
        sta $e9 ; Filename lo
        lda _sed_fname+1
        sta $ea ; Filename hi

        jsr _dosrom  ; enable OverlayRAM

        lda #00
        jsr $dff9  ; verify filename and load file

        ; Get Areas
        lda $c052
        sta _sed_begin ; Start Address Lo
        lda $c053
        sta _sed_begin+1 ; Start Address Hi

        clc
        lda $c04f
        sta _sed_size
        adc _sed_begin
        sta _sed_end ; End Address Lo
        lda $c050
        sta _sed_size+1
        adc _sed_begin+1
        sta _sed_end+1 ; End Address Lo

        jsr _dosrom ; disable OverlayRAM
        ; -- cli ; re-enable interrupts

        ldx _doserr ; Get result (low byte)
        lda #0   ; Get result (high byte)
        rts
.)

_sed_dir
.(
        ; -- sei ; disable interrupts
        jsr _dosrom  ; enable OverlayRAM
        lda #00
        jsr $e344
        jsr _dosrom ; disable OverlayRAM
        ; -- cli ; re-enable interrupts
        ldx _doserr ; Get result (low byte)
        lda #0   ; Get result (high byte)
        rts
.)

uhex    .asc  "0123456789ABCDEF"
uadr    .asc  $22,",A#",0
ustr    .asc  "!STATUS",$22
ubuf    .dsb  32

_sed_update_exe
.(
        lda _sed_fname
        sta nextn+1
        lda _sed_fname+1
        sta nextn+2

        ldx #0
nextn
        lda $1234,x
        beq adr
        sta ubuf,x
        inx
        jmp nextn

adr
        ldy #0
nexta
        lda uadr,y
        beq adrnum
        sta ubuf,x
        inx
        iny
        jmp nexta

adrnum
        ; hi byte
        lda _sed_exe+1
        lsr
        lsr
        lsr
        lsr
        tay
        lda uhex,y
        sta ubuf,x
        inx
        lda _sed_exe+1
        and #$0f
        tay
        lda uhex,y
        sta ubuf,x
        inx

        ; lo byte
        lda _sed_exe
        lsr
        lsr
        lsr
        lsr
        tay
        lda uhex,y
        sta ubuf,x
        inx
        lda _sed_exe
        and #$0f
        tay
        lda uhex,y
        sta ubuf,x
        inx

        lda #0
        sta ubuf,x

        ldx #<ustr
        ldy #>ustr
        jsr execute

        rts
.)

ldr_end
#echo ======================================================
#print (ldr_end-ldr)
#echo ======================================================

; Holds page 0
buf_0000    = (*+$0000)
; Holds page 2
buf_0200    = (*+$0100)
; Holds page 4
buf_0400    = (*+$0200)

; Highest addres used
buf_end     = (*+$0300)

#echo ======================================================
#print (buf_end-ldr)
#echo ======================================================
